/* This file was generated by syncqt. */
#ifndef QT_QTBLUETOOTH_VERSION_H
#define QT_QTBLUETOOTH_VERSION_H

#define QTBLUETOOTH_VERSION_STR "6.11.0"

#define QTBLUETOOTH_VERSION 0x0601100

#endif // QT_QTBLUETOOTH_VERSION_H
