#ifndef QT_FEATURES_Connectivity_src_global_qtconnectivity_config_h_H
#define QT_FEATURES_Connectivity_src_global_qtconnectivity_config_h_H

#define QT_FEATURE_bluetooth 1

#define QT_FEATURE_nfc 1


#endif // QT_FEATURES_Connectivity_src_global_qtconnectivity_config_h_H
