#ifndef QT_FEATURES_Nfc_src_nfc_qtnfc_config_h_H
#define QT_FEATURES_Nfc_src_nfc_qtnfc_config_h_H

#define QT_FEATURE_pcsclite 1

#define QT_FEATURE_neard -1


#endif // QT_FEATURES_Nfc_src_nfc_qtnfc_config_h_H
